""" Non-Deterministic Buchi Automaton (NBA)

A NFA is the tuple: <states, alphabet, tr, init, final>

WARNING: here we represent the alphabet indirectly with the 
set of atomic propositions AP, so alphabet = 2^AP


"""
from __future__ import annotations

import sys
from collections.abc import MutableMapping, Iterator, Iterable, Generator
from typing import Tuple, Optional, Collection, Set
from functools import reduce, partial

from mc308.logics.bool import BoolFormula
from mc308.auto.ts import TS, State
from mc308.auto.nfa import AutTrans
from mc308.auto.nba import NBA


class GNBA(NBA):
    def __init__(self : GNBA,
                 states : Collection[State],
                 tr : Collection[AutTrans],
                 init : Collection[State],
                 final_sets : Collection[Collection[State]],
                 ap : Collection[BoolFormula]):
        super().__init__(states, tr, init, [], ap)

        self.final_sets = set( [frozenset(f) for f in final_sets] )

        # all final sets are from existing states
        assert reduce(lambda x,s : x and s <= self.states, self.final_sets, True)

    def __repr__(self : GNBA) :
        pl = lambda l : ",".join(l)

        tr_list = set()
        for s in self.states:
            for (a,b) in self.tr.get(s, []):
                tr_list.add((s,a,b))

        return """GNBA : {
  states : %s;
  init : %s;
  final_sets : { %s };
  tr : %s;
  ap : %s;
}""" % ( pl([str(l) for l in self.states]),
         pl([str(l) for l in self.init]),
         pl(["{" + pl([str(l) for l in f]) + "}" for f in self.final_sets]),
         pl(["(%s, {%s}, %s)" % (str(a[0]), pl([str(b) for b in a[1]]), str(a[2])) for a in tr_list]),
         pl([str(a) for a in self.ap]),
        )


