"""
Here you will understand how to use the BDD apis from PySMT.

"""


import os
import sys


from mc308.logics.bool import BoolFormula
from mc308.auto.sts import SymbolicTS, from_vmt, get_nexts

from pysmt.shortcuts import (
    get_env,
    Symbol,
    TRUE, FALSE,
    And, Not, Or, Implies, Iff, Xor,
    is_valid, is_sat, Solver
)

from mc308.td8 import (
    kind, itp,
)

def get_ts(fname):
    module_dir = os.path.dirname(os.path.abspath(__file__))
    file_path = os.path.join(module_dir, "td8_models")
    file_path = os.path.join(file_path, fname)
    with open(file_path, 'r') as f:
        sts, invars, live = from_vmt(f)
        return sts

def get_test_cases():
    # Counter modulo 4 must reach all the states
    b0,b1,b2 = Symbol("b0"), Symbol("b1"), Symbol("b2")
    counter = ("counter",
               get_ts("counter.vmt"),
               # invariant properties that are true
               [],
               # invariant properties that are false in ts
               [
                   Not(And(Not(b0), Not(b1))),
                   Not(And(b0, Not(b1))),
                   Not(And(Not(b0), b1)),
                   Not(And(b0, b1)),
               ])

    # Counter modulo 8
    counter8 = ("Counter modulo 8",
                get_ts("counter_8.vmt"),
                [],
                [Not(And(Not(b0), Not(b1), Not(b2))), # all true
                 Not(And(b0, b1, Not(b2))),
                 Not(And(Not(b0), Not(b1), b2)),
                 Not(And(b0, b1, b2)),]
                )

    ex1 = ("Example 1",
           get_ts("ex1.vmt"),
           [Not(b0)], # false
           [Not(And(Not(b0), Not(b1))), # all true
            Not(And(Not(b0), b1)),
            ]
           )

    return [counter,counter8,ex1]


def main():

    print("\nTesting check_invar_kind")
    for test_case in get_test_cases():
        name, ts, false_prop, true_prop = test_case

        methods = [
            ("kind", kind),
            # UNcomment when you're working on interpolation
#            ("itp", itp),
        ]
        for method,mc_algo in methods:
            print("Processing",name)
            for a in false_prop:
                print(method, a.serialize(), "(should be false):", mc_algo(ts, a, 10))
            for a in true_prop:
                print(method, a.serialize(), "(should be true):", mc_algo(ts, a, 10))




if __name__ == "__main__":
    main()

